@@ -1,67 +1,95 @@
-# This file is part of the pyMOR project (http://www.pymor.org).
-# Copyright 2013-2020 pyMOR developers and contributors. All rights reserved.
-# License: BSD 2-Clause License (http://opensource.org/licenses/BSD-2-Clause)
+# This file is part of the pyMOR project (https://www.pymor.org).
+# Copyright 2013-2021 pyMOR developers and contributors. All rights reserved.
+# License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
-import sys, os, re
+import sys
+import os
 import slugify
-
-os.environ['PYMOR_WITH_SPHINX'] = '1'
+import glob
+import sphinx
+from pathlib import Path
 
 # Check Sphinx version
-import sphinx
-if sphinx.__version__ < "1.0.1":
-    raise RuntimeError("Sphinx 1.0.1 or newer required")
+if sphinx.__version__ < "3.4":
+    raise RuntimeError("Sphinx 3.4 or newer required")
 
-needs_sphinx = '1.0'
+needs_sphinx = '3.4'
+os.environ['PYMOR_WITH_SPHINX'] = '1'
 
 # -----------------------------------------------------------------------------
 # General configuration
 # -----------------------------------------------------------------------------
 
-sys.path.insert(0, os.path.abspath('../../src'))
-sys.path.insert(0, os.path.abspath('.'))
-
-#generate autodoc
-import gen_apidoc
-import pymor
-#import pymortests
-import pymordemos
-gen_apidoc.walk(pymor)
-# gen_apidoc.walk(pymortests)
-gen_apidoc.walk(pymordemos)
+this_dir = Path(__file__).resolve().parent
+src_dir = (this_dir / '..' / '..' / 'src').resolve()
+sys.path.insert(0, str(src_dir))
+sys.path.insert(0, str(this_dir))
 
 # Add any Sphinx extension module names here, as strings. They can be extensions
 # coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
 extensions = ['sphinx.ext.autodoc',
               'sphinx.ext.coverage',
               'sphinx.ext.autosummary',
-              'sphinx.ext.viewcode',
+              'sphinx.ext.linkcode',
               'sphinx.ext.intersphinx',
               'pymordocstring',
               'try_on_binder',
-              'jupyter_sphinx',
+              'myst_nb',
               'sphinx.ext.mathjax',
               'sphinx_qt_documentation',
-              'gen_apidoc'
+              'autoapi.extension',
+              'autoapi_pymor',
+              'sphinxcontrib.bibtex',
               ]
+# this enables:
+# substitutions-with-jinja2, direct-latex-math and definition-lists
+# ref: https://myst-parser.readthedocs.io/en/latest/using/syntax-optional.html
+myst_enable_extensions = [
+    "dollarmath",
+    "amsmath",
+    "deflist",
+    "html_image",
+    "colon_fence",
+    "smartquotes",
+    "replacements",
+    "substitution",
+]
+myst_url_schemes = ["http", "https", "mailto"]
+# auto genereated link anchors
+myst_heading_anchors = 2
+import substitutions # noqa
+myst_substitutions = substitutions.myst_substitutions
+jupyter_execute_notebooks = "cache"
+execution_timeout = 120
+# print tracebacks to stdout
+execution_show_tb = True
 
+bibtex_bibfiles = ['bibliography.bib']
 # Add any paths that contain templates here, relative to this directory.
 templates_path = ['_templates']
 
 # The suffix of source filenames.
-source_suffix = '.rst'
+source_suffix = {
+    '.rst': 'restructuredtext',
+    '.ipynb': 'myst-nb',
+    '.md': 'myst-nb',
+}
+
 
 # The master toctree document.
 master_doc = 'index'
 
 # General substitutions.
 project = 'pyMOR'
-copyright = '2013-2020 pyMOR developers and contributors'
+copyright = '2013-2021 pyMOR developers and contributors'
 
 # The default replacements for |version| and |release|, also used in various
 # other places throughout the built documents.
-#
+# imports have to be delayed until after sys.path modification
+import pymor  # noqa
+import autoapi_pymor # noqa
 version = pymor.__version__
+rst_epilog = substitutions.substitutions
 
 # The full version, including alpha/beta/rc tags.
 release = version.split('-')[0]
@@ -69,12 +97,12 @@ print(version, release)
 
 # There are two options for replacing |today|: either, you set today to some
 # non-false value, then it is used:
-#today = ''
+# today = ''
 # Else, today_fmt is used as the format for a strftime call.
 today_fmt = '%B %d, %Y'
 
 # List of documents that shouldn't be included in the build.
-#unused_docs = []
+# unused_docs = []
 
 # The reST default role (used for this markup: `text`) to use for all documents.
 default_role = "literal"
@@ -88,11 +116,11 @@ add_function_parentheses = False
 
 # If true, the current module name will be prepended to all description
 # unit titles (such as .. function::).
-#add_module_names = True
+# add_module_names = True
 
 # If true, sectionauthor and moduleauthor directives will be shown in the
 # output. They are ignored by default.
-#show_authors = False
+# show_authors = False
 
 # The name of the Pygments (syntax highlighting) style to use.
 pygments_style = 'sphinx'
@@ -143,7 +171,7 @@ html_last_updated_fmt = '%b %d, %Y'
 
 # If true, SmartyPants will be used to convert quotes and dashes to
 # typographically correct entities.
-#html_use_smartypants = True
+# html_use_smartypants = True
 
 # Custom sidebar templates, maps document names to template names.
 # all: "**": ["logo-text.html", "globaltoc.html", "localtoc.html", "searchbox.html"]
@@ -152,23 +180,23 @@ html_sidebars = {
 }
 # Additional templates that should be rendered to pages, maps page names to
 # template names.
-#html_additional_pages = {
-    #'index': 'indexcontent.html',
-#}
+# html_additional_pages = {
+#    'index': 'indexcontent.html',
+# }
 
 # If false, no module index is generated.
 html_use_modindex = True
 
 # If true, the reST sources are included in the HTML build as _sources/<name>.
-#html_copy_source = True
+# html_copy_source = True
 
 # If true, an OpenSearch description file will be output, and all pages will
 # contain a <link> tag referring to it.  The value of this option must be the
 # base URL from which the finished HTML is served.
-#html_use_opensearch = ''
+# html_use_opensearch = ''
 
 # If nonempty, this is the file name suffix for HTML files (e.g. ".html").
-#html_file_suffix = '.html'
+# html_file_suffix = '.html'
 
 # Hide link to page source.
 html_show_sourcelink = False
@@ -185,33 +213,33 @@ pngmath_use_preview = True
 # -----------------------------------------------------------------------------
 
 # The paper size ('letter' or 'a4').
-#latex_paper_size = 'letter'
+# latex_paper_size = 'letter'
 
 # The font size ('10pt', '11pt' or '12pt').
-#latex_font_size = '10pt'
+# latex_font_size = '10pt'
 
 # Grouping the document tree into LaTeX files. List of tuples
 # (source start file, target name, title, author, document class [howto/manual]).
-#_stdauthor = 'Written by the NumPy community'
-#latex_documents = [
-  #('reference/index', 'numpy-ref.tex', 'NumPy Reference',
-   #_stdauthor, 'manual'),
-  #('user/index', 'numpy-user.tex', 'NumPy User Guide',
-   #_stdauthor, 'manual'),
-#]
+# _stdauthor = 'Written by the NumPy community'
+# latex_documents = [
+#    ('reference/index', 'numpy-ref.tex', 'NumPy Reference',
+#     _stdauthor, 'manual'),
+#    ('user/index', 'numpy-user.tex', 'NumPy User Guide',
+#     _stdauthor, 'manual'),
+# ]
 
 # The name of an image file (relative to this directory) to place at the top of
 # the title page.
-#latex_logo = None
+# latex_logo = None
 
 # For "manual" documents, if this is true, then toplevel headings are parts,
 # not chapters.
-#latex_use_parts = False
+# latex_use_parts = False
 
 # Additional stuff for the LaTeX preamble.
 
 # Documents to append as an appendix to all manuals.
-#latex_appendices = []
+# latex_appendices = []
 
 # If false, no module index is generated.
 latex_use_modindex = False
@@ -221,7 +249,6 @@ latex_use_modindex = False
 # Autosummary
 # -----------------------------------------------------------------------------
 
-import glob
 autosummary_generate = glob.glob("generated/*.rst")
 
 # -----------------------------------------------------------------------------
@@ -244,12 +271,11 @@ coverage_ignore_c_items = {}
 
 # PyQt5 inventory is only used internally, actual link targets PySide2
 intersphinx_mapping = {'python': ('https://docs.python.org/3', None),
-                       'numpy': ('https://docs.scipy.org/doc/numpy', None),
+                       'numpy': ('https://numpy.org/doc/stable/', None),
                        'PyQt5': ("https://www.riverbankcomputing.com/static/Docs/PyQt5", None),
-                       'scipy': ('https://docs.scipy.org/doc/scipy/reference', None)}
-
-import substitutions
-rst_epilog = substitutions.substitutions
+                       'scipy': ('https://docs.scipy.org/doc/scipy/reference', None),
+                       'matplotlib': ('https://matplotlib.org', None),
+                       'Sphinx': (' https://www.sphinx-doc.org/en/master/', None)}
 
 modindex_common_prefix = ['pymor.']
 
@@ -257,7 +283,25 @@ modindex_common_prefix = ['pymor.']
 # make intersphinx link to pyside2 docs
 qt_documentation = 'PySide2'
 
-branch = os.environ.get('CI_COMMIT_REF_NAME', 'master')
+branch = os.environ.get('CI_COMMIT_REF_NAME', 'main')
 # this must match PYMOR_ROOT/.ci/gitlab/deploy_docs
 try_on_binder_branch = branch.replace('github/PUSH_', 'from_fork__')
 try_on_binder_slug = os.environ.get('CI_COMMIT_REF_SLUG', slugify.slugify(try_on_binder_branch))
+
+
+def linkcode_resolve(domain, info):
+    if domain == 'py':
+        if not info['module']:
+            return None
+        filename = info['module'].replace('.', '/')
+        return f'https://github.com/pymor/pymor/tree/{branch}/src/{filename}.py'
+    return None
+
+
+autoapi_dirs = [src_dir / 'pymor']
+autoapi_type = 'python'
+# allows incremental build
+autoapi_keep_files = True
+autoapi_ignore = ['*/pymordemos/minimal_cpp_demo/*']
+suppress_warnings = ["autoapi"]
+autoapi_template_dir = this_dir / '_templates' / 'autoapi'
