@@ -1,6 +1,6 @@
-# This file is part of the pyMOR project (http://www.pymor.org).
-# Copyright 2013-2020 pyMOR developers and contributors. All rights reserved.
-# License: BSD 2-Clause License (http://opensource.org/licenses/BSD-2-Clause)
+# This file is part of the pyMOR project (https://www.pymor.org).
+# Copyright 2013-2021 pyMOR developers and contributors. All rights reserved.
+# License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
 # flake8: noqa
 
@@ -52,6 +52,7 @@ common = '''
 .. |CacheableObject| replace:: :class:`~pymor.core.cache.CacheableObject`
 
 .. |StationaryProblem| replace:: :class:`~pymor.analyticalproblems.elliptic.StationaryProblem`
+.. |StationaryProblems| replace:: :class:`StationaryProblems <pymor.analyticalproblems.elliptic.StationaryProblem>`
 .. |InstationaryProblem| replace:: :class:`~pymor.analyticalproblems.instationary.InstationaryProblem`
 
 .. |RectDomain| replace:: :class:`~pymor.analyticalproblems.domaindescriptions.RectDomain`
@@ -102,8 +103,8 @@ common = '''
 
 .. |LTIModel| replace:: :class:`~pymor.models.iosys.LTIModel`
 .. |LTIModels| replace:: :class:`LTIModels <pymor.models.iosys.LTIModel>`
-.. |TransferFunction| replace:: :class:`~pymor.models.iosys.TransferFunction`
-.. |TransferFunctions| replace:: :class:`TransferFunctions <pymor.models.iosys.TransferFunction>`
+.. |TransferFunction| replace:: :class:`~pymor.models.transfer_function.TransferFunction`
+.. |TransferFunctions| replace:: :class:`TransferFunctions <pymor.models.transfer_function.TransferFunction>`
 .. |SecondOrderModel| replace:: :class:`~pymor.models.iosys.SecondOrderModel`
 .. |SecondOrderModels| replace:: :class:`SecondOrderModels <pymor.models.iosys.SecondOrderModel>`
 .. |LinearDelayModel| replace:: :class:`~pymor.models.iosys.LinearDelayModel`
@@ -151,3 +152,29 @@ common = '''
 '''
 
 substitutions = interfaces + common
+
+# list of (key, jinja_safe_key, substitution_value)
+jinja_subst = []
+for line in substitutions.split('\n'):
+    if line == "":
+        continue
+    key, subst = line.split(' replace:: ')
+    key = key.strip()
+    key = key.replace('.. |', '').replace('|', '')
+    subst = subst.replace(':', '{', 1).replace(':', '}', 2)
+    jinja_subst.append((key, key.replace(' ', '_'), subst))
+
+inline_directives = ['math', 'meth', 'class', 'ref', 'mod', 'attr', 'doc', ]
+
+if __name__ == '__main__':
+    import pprint as pp
+    # pp.pprint(jinja_subst)
+    with open('rst_to_myst.sed', 'wt') as out:
+        for dr in inline_directives:
+            out.write(f's;:{dr}:;{{{dr}}};g\n')
+        l = '{{\\ '
+        r = '\\ }}'
+        for key, safe_key, _ in jinja_subst:
+            out.write(f's;|{key}|;{l}{safe_key}{r};g\n')
+
+myst_substitutions = {safe_key: subst for _, safe_key, subst in jinja_subst}
