@@ -1,6 +1,6 @@
-# This file is part of the pyMOR project (http://www.pymor.org).
-# Copyright 2013-2020 pyMOR developers and contributors. All rights reserved.
-# License: BSD 2-Clause License (http://opensource.org/licenses/BSD-2-Clause)
+# This file is part of the pyMOR project (https://www.pymor.org).
+# Copyright 2013-2021 pyMOR developers and contributors. All rights reserved.
+# License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
 from fractions import Fraction
 
@@ -22,7 +22,7 @@ def adaptive_weak_greedy(surrogate, parameter_space, target_error=None, max_exte
 
     This method extends pyMOR's default :func:`~pymor.algorithms.greedy.weak_greedy`
     greedy basis generation algorithm by adaptive refinement of the
-    parameter training set according to [HDO11]_ to prevent overfitting
+    parameter training set according to :cite:`HDO11` to prevent overfitting
     of the approximation basis to the training set. This is achieved by
     estimating the approximation error on an additional validation set of
     parameters. If the ratio between the estimated errors on the validation
@@ -78,7 +78,6 @@ def adaptive_weak_greedy(surrogate, parameter_space, target_error=None, max_exte
         :training_set_sizes:     The final size of the training set in each extension step.
         :time:                   Duration of the algorithm.
     """
-
     logger = getLogger('pymor.algorithms.adaptivegreedy.adaptive_weak_greedy')
 
     if pool is None or pool is dummy_pool:
@@ -196,7 +195,7 @@ def adaptive_weak_greedy(surrogate, parameter_space, target_error=None, max_exte
         refinements.append(current_refinements)
         training_set_sizes.append(len(sample_set.vertex_mus))
 
-        # break if traget error reached
+        # break if target error reached
         if target_error is not None and max_err <= target_error:
             logger.info(f'Reached maximal error on snapshots of {max_err} <= {target_error}')
             break
@@ -235,7 +234,7 @@ def rb_adaptive_greedy(fom, reductor, parameter_space,
 
     This method extends pyMOR's default :func:`~pymor.algorithms.greedy.rb_greedy`
     greedy reduced basis generation algorithm by adaptive refinement of the
-    parameter training set [HDO11]_ to prevent overfitting
+    parameter training set :cite:`HDO11` to prevent overfitting
     of the reduced basis to the training set as implemented in :func:`adaptive_weak_greedy`.
 
     Parameters
@@ -286,7 +285,6 @@ def rb_adaptive_greedy(fom, reductor, parameter_space,
         :training_set_sizes:     The final size of the training set in each extension step.
         :time:                   Duration of the algorithm.
     """
-
     surrogate = RBSurrogate(fom, reductor, use_error_estimator, error_norm, extension_params, pool or dummy_pool)
 
     result = adaptive_weak_greedy(surrogate, parameter_space, target_error=target_error, max_extensions=max_extensions,
