@@ -1,8 +1,8 @@
-# This file is part of the pyMOR project (http://www.pymor.org).
-# Copyright 2013-2020 pyMOR developers and contributors. All rights reserved.
-# License: BSD 2-Clause License (http://opensource.org/licenses/BSD-2-Clause)
+# This file is part of the pyMOR project (https://www.pymor.org).
+# Copyright 2013-2021 pyMOR developers and contributors. All rights reserved.
+# License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
-""" This module provides some operators for finite volume discretizations."""
+"""This module provides some operators for finite volume discretizations."""
 from functools import partial
 
 import numpy as np
@@ -20,14 +20,12 @@ from pymor.discretizers.builtin.grids.interfaces import GridWithOrthogonalCenter
 from pymor.discretizers.builtin.grids.referenceelements import line, triangle, square
 from pymor.discretizers.builtin.grids.subgrid import SubGrid, make_sub_grid_boundary_info
 from pymor.discretizers.builtin.gui.visualizers import PatchVisualizer, OnedVisualizer
-from pymor.discretizers.builtin.inplace import iadd_masked, isub_masked
 from pymor.discretizers.builtin.quadratures import GaussQuadratures
 from pymor.models.basic import StationaryModel, InstationaryModel
 from pymor.operators.constructions import ComponentProjectionOperator, LincombOperator, ZeroOperator
 from pymor.operators.interface import Operator
 from pymor.operators.numpy import NumpyGenericOperator, NumpyMatrixBasedOperator, NumpyMatrixOperator
 from pymor.parameters.base import ParametricObject
-from pymor.tools.deprecated import Deprecated
 from pymor.vectorarrays.numpy import NumpyVectorSpace
 
 
@@ -268,7 +266,7 @@ class NonlinearAdvectionOperator(Operator):
             self._fetch_grid_data()
 
         U = U.to_numpy()
-        R = np.zeros((len(U), self.source.dim))
+        R = np.zeros((len(U), self.source.dim + 1))
 
         bi = self.boundary_info
         gd = self._grid_data
@@ -308,12 +306,12 @@ class NonlinearAdvectionOperator(Operator):
             if bi.has_neumann:
                 NUM_FLUX[NEUMANN_BOUNDARIES] = 0
 
-            iadd_masked(Ri, NUM_FLUX, SUPE[:, 0])
-            isub_masked(Ri, NUM_FLUX, SUPE[:, 1])
+            np.add.at(Ri, SUPE[:, 0], NUM_FLUX)
+            np.subtract.at(Ri, SUPE[:, 1], NUM_FLUX)
 
-        R /= VOLS0
+        R[:, :-1] /= VOLS0
 
-        return self.range.make_array(R)
+        return self.range.make_array(R[:, :-1])
 
     def jacobian(self, U, mu=None):
         assert U in self.source and len(U) == 1
@@ -435,8 +433,7 @@ def nonlinear_advection_lax_friedrichs_operator(grid, boundary_info, flux, lxf_l
 
 def nonlinear_advection_simplified_engquist_osher_operator(grid, boundary_info, flux, flux_derivative,
                                                            dirichlet_data=None, solver_options=None, name=None):
-    """Instantiate a :class:`NonlinearAdvectionOperator` using
-    :class:`SimplifiedEngquistOsherFlux`."""
+    """Create a :class:`NonlinearAdvectionOperator` using :class:`SimplifiedEngquistOsherFlux`."""
     num_flux = SimplifiedEngquistOsherFlux(flux, flux_derivative)
     return NonlinearAdvectionOperator(grid, boundary_info, num_flux, dirichlet_data, solver_options, name=name)
 
@@ -486,8 +483,8 @@ class LinearAdvectionLaxFriedrichsOperator(NumpyMatrixBasedOperator):
         edge_volumes = g.volumes(1)
         boundary_edges = g.boundaries(1)
         inner_edges = np.setdiff1d(np.arange(g.size(1)), boundary_edges)
-        dirichlet_edges = bi.dirichlet_boundaries(1) if bi.has_dirichlet else np.array([], ndmin=1, dtype=np.int)
-        neumann_edges = bi.neumann_boundaries(1) if bi.has_neumann else np.array([], ndmin=1, dtype=np.int)
+        dirichlet_edges = bi.dirichlet_boundaries(1) if bi.has_dirichlet else np.array([], ndmin=1, dtype=int)
+        neumann_edges = bi.neumann_boundaries(1) if bi.has_neumann else np.array([], ndmin=1, dtype=int)
         outflow_edges = np.setdiff1d(boundary_edges, np.hstack([dirichlet_edges, neumann_edges]))
         normal_velocities = np.einsum('ei,ei->e',
                                       self.velocity_field(g.centers(1), mu=mu),
@@ -518,11 +515,6 @@ class LinearAdvectionLaxFriedrichsOperator(NumpyMatrixBasedOperator):
         return A
 
 
-@Deprecated(LinearAdvectionLaxFriedrichsOperator)
-def LinearAdvectionLaxFriedrichs(*args, **kwargs):
-    return LinearAdvectionLaxFriedrichsOperator(*args, **kwargs)
-
-
 class L2Product(NumpyMatrixBasedOperator):
     """|Operator| representing the L2-product between finite volume functions.
 
@@ -610,7 +602,7 @@ class NonlinearReactionOperator(Operator):
         return NumpyMatrixOperator(A, source_id=self.source.id, range_id=self.range.id)
 
 
-class L2ProductFunctional(NumpyMatrixBasedOperator):
+class L2Functional(NumpyMatrixBasedOperator):
     """Finite volume functional representing the inner product with an L2-|Function|.
 
     Additionally, boundary conditions can be enforced by providing `dirichlet_data`
@@ -704,6 +696,51 @@ class L2ProductFunctional(NumpyMatrixBasedOperator):
         return F_INTS.reshape((-1, 1))
 
 
+class BoundaryL2Functional(NumpyMatrixBasedOperator):
+    """FV functional representing the inner product with an L2-|Function| on the boundary.
+
+    Parameters
+    ----------
+    grid
+        |Grid| for which to assemble the functional.
+    function
+        The |Function| with which to take the inner product.
+    boundary_type
+        The type of domain boundary (e.g. 'neumann') on which to assemble the functional.
+        If `None` the functional is assembled over the whole boundary.
+    boundary_info
+        If `boundary_type` is specified, the
+        |BoundaryInfo| determining which boundary entity belongs to which physical boundary.
+    name
+        The name of the functional.
+    """
+
+    sparse = False
+    source = NumpyVectorSpace(1)
+
+    def __init__(self, grid, function, boundary_type=None, boundary_info=None, name=None):
+        assert grid.reference_element(0) in {line, triangle, square}
+        assert function.shape_range == ()
+        assert not boundary_type or boundary_info
+        self.__auto_init(locals())
+        self.range = FVVectorSpace(grid)
+
+    def _assemble(self, mu=None):
+        g = self.grid
+
+        NI = self.boundary_info.boundaries(self.boundary_type, 1) if self.boundary_type else g.boundaries(1)
+        SUPE = g.superentities(1, 0)[NI][:, 0]
+        F = self.function(g.centers(1)[NI], mu=mu)
+        if g.dim == 1:
+            I = np.zeros(self.range.dim)
+            I[SUPE] = F
+        else:
+            I = F*g.integration_elements(1)[NI]
+            I = coo_matrix((I, (np.zeros_like(SUPE), SUPE)), shape=(1, g.size(0))).toarray().ravel()
+
+        return I.reshape((-1, 1))
+
+
 class DiffusionOperator(NumpyMatrixBasedOperator):
     """Finite Volume Diffusion |Operator|.
 
@@ -814,6 +851,34 @@ class DiffusionOperator(NumpyMatrixBasedOperator):
         return A
 
 
+class InterpolationOperator(NumpyMatrixBasedOperator):
+    """Vector-like L^2-projection interpolation |Operator| for finite volume spaces.
+
+    Parameters
+    ----------
+    grid
+        The |Grid| on which to interpolate.
+    function
+        The |Function| to interpolate.
+    order
+        The quadrature order to compute the element-wise averages
+    """
+
+    source = NumpyVectorSpace(1)
+    linear = True
+
+    def __init__(self, grid, function, order=0):
+        assert function.dim_domain == grid.dim
+        assert function.shape_range == ()
+        if order != 0:
+            raise NotImplementedError('Higher orders not implemented yet!')
+        self.__auto_init(locals())
+        self.range = FVVectorSpace(grid)
+
+    def _assemble(self, mu=None):
+        return self.function.evaluate(self.grid.centers(0), mu=mu).reshape((-1, 1))
+
+
 def discretize_stationary_fv(analytical_problem, diameter=None, domain_discretizer=None, grid_type=None,
                              num_flux='lax_friedrichs', lxf_lambda=1., eo_gausspoints=5, eo_intervals=1,
                              grid=None, boundary_info=None, preassemble=True):
@@ -866,7 +931,6 @@ def discretize_stationary_fv(analytical_problem, diameter=None, domain_discretiz
             :unassembled_m:  In case `preassemble` is `True`, the generated |Model|
                              before preassembling operators.
     """
-
     assert isinstance(analytical_problem, StationaryProblem)
     assert grid is None or boundary_info is not None
     assert boundary_info is None or grid is not None
@@ -877,8 +941,6 @@ def discretize_stationary_fv(analytical_problem, diameter=None, domain_discretiz
 
     if analytical_problem.robin_data is not None:
         raise NotImplementedError
-    if p.outputs:
-        raise NotImplementedError
 
     if grid is None:
         domain_discretizer = domain_discretizer or discretize_domain_default
@@ -893,7 +955,7 @@ def discretize_stationary_fv(analytical_problem, diameter=None, domain_discretiz
     F, F_coefficients = [], []
 
     if p.rhs is not None or p.neumann_data is not None:
-        F += [L2ProductFunctional(grid, p.rhs, boundary_info=boundary_info, neumann_data=p.neumann_data)]
+        F += [L2Functional(grid, p.rhs, boundary_info=boundary_info, neumann_data=p.neumann_data)]
         F_coefficients += [1.]
 
     # diffusion part
@@ -902,8 +964,8 @@ def discretize_stationary_fv(analytical_problem, diameter=None, domain_discretiz
               for i, df in enumerate(p.diffusion.functions)]
         L_coefficients += p.diffusion.coefficients
         if p.dirichlet_data is not None:
-            F += [L2ProductFunctional(grid, None, boundary_info=boundary_info, dirichlet_data=p.dirichlet_data,
-                                      diffusion_function=df, name=f'dirichlet_{i}')
+            F += [L2Functional(grid, None, boundary_info=boundary_info, dirichlet_data=p.dirichlet_data,
+                               diffusion_function=df, name=f'dirichlet_{i}')
                   for i, df in enumerate(p.diffusion.functions)]
             F_coefficients += p.diffusion.coefficients
 
@@ -911,8 +973,8 @@ def discretize_stationary_fv(analytical_problem, diameter=None, domain_discretiz
         L += [DiffusionOperator(grid, boundary_info, diffusion_function=p.diffusion, name='diffusion')]
         L_coefficients += [1.]
         if p.dirichlet_data is not None:
-            F += [L2ProductFunctional(grid, None, boundary_info=boundary_info, dirichlet_data=p.dirichlet_data,
-                                      diffusion_function=p.diffusion, name='dirichlet')]
+            F += [L2Functional(grid, None, boundary_info=boundary_info, dirichlet_data=p.dirichlet_data,
+                               diffusion_function=p.diffusion, name='dirichlet')]
             F_coefficients += [1.]
 
     # advection part
@@ -944,7 +1006,9 @@ def discretize_stationary_fv(analytical_problem, diameter=None, domain_discretiz
 
     # reaction part
     if isinstance(p.reaction, LincombFunction):
-        raise NotImplementedError
+        L += [ReactionOperator(grid, rf, name=f'reaction_{i}')
+              for i, rf in enumerate(p.reaction.functions)]
+        L_coefficients += list(p.reaction.coefficients)
     elif p.reaction is not None:
         L += [ReactionOperator(grid, p.reaction, name='reaction')]
         L_coefficients += [1.]
@@ -969,7 +1033,7 @@ def discretize_stationary_fv(analytical_problem, diameter=None, domain_discretiz
         F = LincombOperator(operators=F, coefficients=F_coefficients, name='rhs')
 
     if grid.reference_element in (triangle, square):
-        visualizer = PatchVisualizer(grid=grid, bounding_box=grid.bounding_box(), codim=0)
+        visualizer = PatchVisualizer(grid=grid, codim=0)
     elif grid.reference_element is line:
         visualizer = OnedVisualizer(grid=grid, codim=0)
     else:
@@ -978,7 +1042,37 @@ def discretize_stationary_fv(analytical_problem, diameter=None, domain_discretiz
     l2_product = L2Product(grid, name='l2')
     products = {'l2': l2_product}
 
-    m = StationaryModel(L, F, products=products, visualizer=visualizer,
+    # assemble additional output functionals
+    if p.outputs:
+        if any(v[0] not in ('l2', 'l2_boundary') for v in p.outputs):
+            raise NotImplementedError
+        outputs = []
+        for v in p.outputs:
+            if v[0] == 'l2':
+                if isinstance(v[1], LincombFunction):
+                    ops = [L2Functional(grid, vv).H
+                           for vv in v[1].functions]
+                    outputs.append(LincombOperator(ops, v[1].coefficients))
+                else:
+                    outputs.append(L2Functional(grid, v[1]).H)
+            else:
+                if isinstance(v[1], LincombFunction):
+                    ops = [BoundaryL2Functional(grid, vv).H
+                           for vv in v[1].functions]
+                    outputs.append(LincombOperator(ops, v[1].coefficients))
+                else:
+                    outputs.append(BoundaryL2Functional(grid, v[1]).H)
+        if len(outputs) > 1:
+            from pymor.operators.block import BlockColumnOperator
+            from pymor.operators.constructions import NumpyConversionOperator
+            output_functional = BlockColumnOperator(outputs)
+            output_functional = NumpyConversionOperator(output_functional.range) @ output_functional
+        else:
+            output_functional = outputs[0]
+    else:
+        output_functional = None
+
+    m = StationaryModel(L, F, products=products, output_functional=output_functional, visualizer=visualizer,
                         name=f'{p.name}_FV')
 
     data = {'grid': grid, 'boundary_info': boundary_info}
@@ -994,8 +1088,7 @@ def discretize_instationary_fv(analytical_problem, diameter=None, domain_discret
                                num_flux='lax_friedrichs', lxf_lambda=1., eo_gausspoints=5, eo_intervals=1,
                                grid=None, boundary_info=None, num_values=None, time_stepper=None, nt=None,
                                preassemble=True):
-    """Discretizes an |InstationaryProblem| with a |StationaryProblem| as stationary part
-    using the finite volume method.
+    """FV Discretization of an |InstationaryProblem| with a |StationaryProblem| as stationary part
 
     Parameters
     ----------
@@ -1054,7 +1147,6 @@ def discretize_instationary_fv(analytical_problem, diameter=None, domain_discret
             :unassembled_m:  In case `preassemble` is `True`, the generated |Model|
                              before preassembling operators.
     """
-
     assert isinstance(analytical_problem, InstationaryProblem)
     assert isinstance(analytical_problem.stationary_part, StationaryProblem)
     assert grid is None or boundary_info is not None
@@ -1092,7 +1184,9 @@ def discretize_instationary_fv(analytical_problem, diameter=None, domain_discret
     rhs = None if isinstance(m.rhs, ZeroOperator) else m.rhs
 
     m = InstationaryModel(operator=m.operator, rhs=rhs, mass=None, initial_data=I, T=p.T,
-                          products=m.products, time_stepper=time_stepper,
+                          products=m.products,
+                          output_functional=m.output_functional,
+                          time_stepper=time_stepper,
                           visualizer=m.visualizer,
                           num_values=num_values, name=f'{p.name}_FV')
 
