@@ -1,6 +1,6 @@
-# This file is part of the pyMOR project (http://www.pymor.org).
-# Copyright 2013-2020 pyMOR developers and contributors. All rights reserved.
-# License: BSD 2-Clause License (http://opensource.org/licenses/BSD-2-Clause)
+# This file is part of the pyMOR project (https://www.pymor.org).
+# Copyright 2013-2021 pyMOR developers and contributors. All rights reserved.
+# License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
 
 from pymor.core.base import ImmutableObject
@@ -22,10 +22,10 @@ class PatchVisualizer(ImmutableObject):
     ----------
     grid
         The underlying |Grid|.
-    bounding_box
-        A bounding box in which the grid is contained.
     codim
         The codimension of the entities the data in `U` is attached to (either 0 or 2).
+    bounding_box
+        A bounding box in which the grid is contained.
     backend
         Plot backend to use ('jupyter_or_gl', 'jupyter', 'gl', 'matplotlib').
     block
@@ -33,16 +33,18 @@ class PatchVisualizer(ImmutableObject):
     """
 
     @defaults('backend')
-    def __init__(self, grid, bounding_box=([0, 0], [1, 1]), codim=2, backend='jupyter_or_gl', block=False):
+    def __init__(self, grid, codim=2, bounding_box=None, backend='jupyter_or_gl', block=False):
         assert grid.reference_element in (triangle, square)
         assert grid.dim == 2
         assert codim in (0, 2)
         assert backend in {'jupyter_or_gl', 'jupyter', 'gl', 'matplotlib'}
         if backend == 'jupyter_or_gl':
             backend = 'jupyter' if is_jupyter() else 'gl'
+        if bounding_box is None:
+            bounding_box = grid.bounding_box()
         self.__auto_init(locals())
 
-    def visualize(self, U, m, title=None, legend=None, separate_colorbars=False,
+    def visualize(self, U, title=None, legend=None, separate_colorbars=False,
                   rescale_colorbars=False, block=None, filename=None, columns=2):
         """Visualize the provided data.
 
@@ -53,8 +55,6 @@ class PatchVisualizer(ImmutableObject):
             as a time series of plots. Alternatively, a tuple of |VectorArrays| can be
             provided, in which case a subplot is created for each entry of the tuple. The
             lengths of all arrays have to agree.
-        m
-            Filled in by :meth:`pymor.models.interface.Model.visualize` (ignored).
         title
             Title of the plot.
         legend
@@ -126,7 +126,7 @@ class OnedVisualizer(ImmutableObject):
             backend = 'jupyter' if is_jupyter() else 'matplotlib'
         self.__auto_init(locals())
 
-    def visualize(self, U, m, title=None, legend=None, separate_plots=True,
+    def visualize(self, U, title=None, legend=None, separate_plots=True,
                   separate_axes=False, block=None, filename=None, columns=2):
         """Visualize the provided data.
 
@@ -137,8 +137,6 @@ class OnedVisualizer(ImmutableObject):
             as a time series of plots. Alternatively, a tuple of |VectorArrays| can be
             provided, in which case several plots are made into the same axes. The
             lengths of all arrays have to agree.
-        m
-            Filled in by :meth:`pymor.models.interface.Model.visualize` (ignored).
         title
             Title of the plot.
         legend
